/* FeaturePredicateConjunction.java
 *
 * created: Tue Aug  6 2002
 *
 * This file is part of Artemis
 *
 * Copyright (C) 2001  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/pathsoft/repository/powmap/diana/FeaturePredicateConjunction.java,v 1.2 2002/10/25 08:49:21 kmr Exp $
 */

package diana;

/**
 *  A class for combining two predicates.
 *
 *  @author Kim Rutherford <kmr@sanger.ac.uk>
 *  @version $Id: FeaturePredicateConjunction.java,v 1.2 2002/10/25 08:49:21 kmr Exp $
 **/

public class FeaturePredicateConjunction implements FeaturePredicate {
  /**
   *  Create a new FeaturePredicateConjunction object.
   *  @param type the type of conjunction to use when testPredicate() is
   *    called.
   **/
  public FeaturePredicateConjunction (final FeaturePredicate predicate1,
                                      final FeaturePredicate predicate2,
                                      final int type) {
    this.predicate1 = predicate1;
    this.predicate2 = predicate2;
    this.type = type;

    if (!(type == OR || type == AND)) {
      throw new Error ("internal error - illegal type given to " +
                       "FeaturePredicateConjunction constructor");
    }
  }

  /**
   *  The && predicate type.
   **/
  final public static int AND = 1;

  /**
   *  The || predicate type.
   **/
  final public static int OR = 0;

  /**
   *  Test the given Feature against this predicate.
   *  @param feature The Feature to test the predicate against.
   *  @return predicate1 && predicate2 if AND was passed as a type to the
   *    constructor.  Returns predicate1 || predicate2 if OR was passed to the
   *    constructor.
   **/
  public boolean testPredicate (final Feature feature) {
    if (type == AND) {
      return (predicate1.testPredicate (feature) &&
              predicate2.testPredicate (feature));
    } else {
      return (predicate1.testPredicate (feature) &&
              predicate2.testPredicate (feature));
    }
  }

  /**
   *  The first FeaturePredicate passed to the constructor.
   **/
  private FeaturePredicate predicate1;

  /**
   *  The second FeaturePredicate passed to the constructor.
   **/
  private FeaturePredicate predicate2;

  /**
   *  The type of conjunction passed to the constructor.
   **/
  private int type;
}
