/* ChoiceFrame.java
 *
 * created: Tue Aug  6 2002
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 2001  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/pathsoft/repository/powmap/diana/components/ChoiceFrame.java,v 1.2 2002/08/07 08:11:11 kmr Exp $
 */

package diana.components;

import uk.ac.sanger.pathogens.StringVector;

import java.awt.*;
import java.awt.event.*;

/**
 *  A Choice in a Frame.
 *
 *  @author Kim Rutherford <kmr@sanger.ac.uk>
 *  @version $Id: ChoiceFrame.java,v 1.2 2002/08/07 08:11:11 kmr Exp $
 **/

public class ChoiceFrame extends Frame {
  /**
   *  Create a new ChoiceFrame component with the given list of Strings.
   **/
  public ChoiceFrame (final StringVector strings) {
    choice = new Choice ();

    for (int i = 0 ; i < strings.size () ; ++i) {
      choice.add (strings.elementAt (i));
    }

    final Panel choice_panel = new Panel ();
    choice_panel.add (choice);
    
    add (choice_panel, "Center");

    final Panel panel = new Panel ();

    panel.add (ok_button);
    ok_button.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ChoiceFrame.this.dispose ();
      }
    });

    panel.add (close_button);
    close_button.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent e) {
        ChoiceFrame.this.dispose ();
      }
    });

    add (panel, "South");
    pack ();

    addWindowListener (new WindowAdapter () {
      public void windowClosing (WindowEvent event) {
        ChoiceFrame.this.dispose ();
      }
    });

    pack ();

    final Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation (new Point ((screen.width - getSize ().width) / 2,
                            (screen.height - getSize ().height) / 2));
  }

  /**
   *  Return the Choice component that is displayed in this Frame.
   **/
  public Choice getChoice () {
    return choice;
  }

  /**
   *  Return the reference of the OK button of this Chooser.
   **/
  public Button getOKButton () {
    return ok_button;
  }

  private Choice choice;

  final private Button ok_button = new Button ("OK");

  final private Button close_button = new Button ("Cancel");
}
